;****************************************************************
;*  TR099.ASM
;*			Zeitsteuerung fr Junkers-Therme
;*			================================
;*
;*  22.11.1998	Bodo Wenzel	V1.0, Erstellung
;*  2001-04-02	Bodo Wenzel	V1.1, Modifikationen:
;*	Wiederholung des LCD-Inits bei Tastenbettigung wegen
;*	Verdacht auf dessen Empfindlichkeit.
;*	Timeout fr Eingaben auerhalb der Hauptschleife.
;*	Korrekter Zustand, wenn Zeit neu gesetzt wird.
;*	Kein Lschen der Uhrzeit mehr bei Resets.
;****************************************************************

	title		"Junkers-Zeitsteuerung"

VERSION	equ	H'11'

;================================================================
;	Definitionen

	processor	16c84
	radix		dec
	include		"p16cxx.inc"

;================================================================
;	Konfiguration

	org	H'2007'
	dw	_CP_OFF & _PWRTE_ON & _WDT_OFF & _XT_OSC

;================================================================
;	Konstanten

F_OSC		equ	2457600		;Quarzfrequenz [Hz]

F_LCD		equ	100		;"Bild"-Frequenz LCD

LCD_DSP_ADDR	equ	H'00'
LCD_DIV_ADDR	equ	H'41'		;Adressen im LCD

T_TIMEOUT	equ	15		;Eingabetimeout [s]

;================================================================
;	Ein- und Ausgnge

TRIS_A	equ	B'00000000'	;Port A:
LCD_DAT	equ	0		;O LCD Daten
LCD_CLK	equ	1		;O LCD Takt
MUX	equ	2		;O Umschalter 0/1 = Tag/Nacht
;	equ	3		;O unbenutzt
;	equ	4		;O unbenutzt

TRIS_B	equ	B'00001111'	;Port B:
KEY_ESC	equ	0		;I Taste
KEY_DN	equ	1		;I Taste
KEY_OK	equ	2		;I Taste
KEY_UP	equ	3		;I Taste
;	equ	4		;O unbenutzt
;	equ	5		;O unbenutzt
;	equ	6		;O unbenutzt (ISP)
;	equ	7		;O unbenutzt (ISP)

KEY_ALL	equ	(1<<KEY_ESC)|(1<<KEY_UP)|(1<<KEY_DN)|(1<<KEY_OK)

OPT	equ	B'01010100'	;Pullups an Port B
				;TMR0 als Timer F_CYCLE/32
F_TMR0	equ	F_OSC/4/32

N_TICKS	equ	F_TMR0/256	;Ticks pro Sekunde
	if	N_TICKS>256 || N_TICKS<10
	error	"F_TMR0 falsch"
	endif

;================================================================
;	Daten

	org	H'0C'

flags:		res	1	;Diverse Flags (siehe fl_...)
fl_action	equ	0	;Reaktion des Programms ntig
fl_blink0	equ	1
fl_blink1	equ	2
fl_blink2	equ	3
fl_blink3	equ	4	;Blinken
fl_sw		equ	5	;Schaltzeit wird gendert
fl_lcd_par	equ	6	;Paritt fr LCD-Ausgabe
fl_colon	equ	7	;Doppelpunkt bei Zeitausgabe

NO_BLINK	set	~(1<<fl_blink0) & ~(1<<fl_blink1)
NO_BLINK	set	NO_BLINK & ~(1<<fl_blink2)
NO_BLINK	set	NO_BLINK & ~(1<<fl_blink3)

int_w:		res	1
int_status:	res	1	;Puffer whrend Interrupt

keys:		res	2	;Tasten (entprellt, neu/alt)

ticks:		res	1
second:		res	1
minute:		res	1
hour:		res	1
day:		res	1	;aktuelle Zeit

timeout:	res	1	;Eingabetimeout

sw_minute:	res	1
sw_hour:	res	1
sw_day:		res	1	;nchste Schaltzeit

sw_old:		res	2	;gerettete Schaltzeiger

edit_buf:	res	3
edit_min:	res	2
edit_max:	res	2	;fr nderungen

dsp_buf:	res	2
dsp_cnt:	res	1
b2b_buf:	res	1	;fr Umwandlung/Ausgabe

lcd_line:	res	4
lcd_cnt:	res	1
lcd_addr:	res	1
lcd_buf:	res	1	;fr LCD-Ausgabe

	if	$ > H'30'
	error	"Zu viel RAM benutzt"
	endif

;================================================================
;	Vektoren

	org	H'0000'
	goto	reset

	dw	VERSION

	org	H'0004'		;geht ohne Sprung weiter...
;================================================================
;	Interrupts
interrupt:
	movwf	int_w
	swapf	STATUS,w
	bcf	STATUS,RP0
	movwf	int_status	;alles retten

	bcf	INTCON,T0IF	;Timer zurcksetzen
;----------------------------------------------------------------
	movf	PORTB,w
	andlw	KEY_ALL
	movwf	keys		;Tasten (entprellt) lesen

	xorwf	keys+1,w
	btfss	STATUS,Z
	bsf	flags,fl_action
	xorwf	keys+1		;Wechsel melden
;----------------------------------------------------------------
	decfsz	ticks
	goto	i_quit		;noch keine Sekunde?
	movlw	N_TICKS
	movwf	ticks

	bsf	flags,fl_action	;Aktion ntig

	decfsz	timeout,w
	movwf	timeout		;Eingabetimeout bearbeiten

	decfsz	second
	goto	i_quit		;noch keine Minute?
	movlw	60
	movwf	second

	incf	minute
	movf	minute,w
	xorlw	60
	btfss	STATUS,Z
	goto	i_quit		;noch keine Stunde?
	clrf	minute

	incf	hour
	movf	hour,w
	xorlw	24
	btfss	STATUS,Z
	goto	i_quit		;noch kein Tag?
	clrf	hour

	incf	day
	movf	day,w
	xorlw	7
	btfsc	STATUS,Z
	clrf	day		;Woche um?
;----------------------------------------------------------------
i_quit:
	swapf	int_status,w
	movwf	STATUS
	swapf	int_w
	swapf	int_w,w		;alles wiederholen
	retfie
;================================================================
;	Zeichensatz

gp_pat	macro	pattern
gp_bits	set	           (pattern&B'10000000')
gp_bits	set	gp_bits | ((pattern&B'01000000')>>1)
gp_bits	set	gp_bits | ((pattern&B'00100000')>>2)
gp_bits	set	gp_bits | ((pattern&B'00010000')>>3)
gp_bits	set	gp_bits | ((pattern&B'00001000')<<3)
gp_bits	set	gp_bits | ((pattern&B'00000100')<<2)
gp_bits	set	gp_bits | ((pattern&B'00000010')<<1)
gp_bits	set	gp_bits |  (pattern&B'00000001')
	retlw	gp_bits
	endm

get_pattern:
	addwf	PCL		;Bitmuster holen

	gp_pat	B'00000000'	;0
	gp_pat	B'00111100'
	gp_pat	B'01100110'
	gp_pat	B'01100110'
	gp_pat	B'01100110'
	gp_pat	B'01100110'
	gp_pat	B'01100110'
	gp_pat	B'00111100'

	gp_pat	B'01100011'	;M
	gp_pat	B'01110111'
	gp_pat	B'01111111'
	gp_pat	B'01101011'
	gp_pat	B'01100011'
	gp_pat	B'01100011'
	gp_pat	B'01100011'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;1
	gp_pat	B'00001000'
	gp_pat	B'00011000'
	gp_pat	B'00111000'
	gp_pat	B'00011000'
	gp_pat	B'00011000'
	gp_pat	B'00011000'
	gp_pat	B'00111100'

	gp_pat	B'01111100'	;D
	gp_pat	B'01100110'
	gp_pat	B'01100011'
	gp_pat	B'01100011'
	gp_pat	B'01100011'
	gp_pat	B'01100110'
	gp_pat	B'01111100'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;2
	gp_pat	B'00111100'
	gp_pat	B'01100110'
	gp_pat	B'00001100'
	gp_pat	B'00011000'
	gp_pat	B'00110000'
	gp_pat	B'01100000'
	gp_pat	B'01111110'

	gp_pat	B'01111111'	;F
	gp_pat	B'01100000'
	gp_pat	B'01100000'
	gp_pat	B'01111100'
	gp_pat	B'01100000'
	gp_pat	B'01100000'
	gp_pat	B'01100000'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;3
	gp_pat	B'00111100'
	gp_pat	B'01100110'
	gp_pat	B'00000110'
	gp_pat	B'00011100'
	gp_pat	B'00000110'
	gp_pat	B'01100110'
	gp_pat	B'00111100'

	gp_pat	B'00111110'	;S
	gp_pat	B'01100011'
	gp_pat	B'01100000'
	gp_pat	B'00111110'
	gp_pat	B'00000011'
	gp_pat	B'01100011'
	gp_pat	B'00111110'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;4
	gp_pat	B'00000100'
	gp_pat	B'00001100'
	gp_pat	B'00011100'
	gp_pat	B'00101100'
	gp_pat	B'01101100'
	gp_pat	B'01111110'
	gp_pat	B'00001100'

	gp_pat	B'00000000'	;o
	gp_pat	B'00000000'
	gp_pat	B'00011100'
	gp_pat	B'00110110'
	gp_pat	B'00110110'
	gp_pat	B'00110110'
	gp_pat	B'00011100'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;5
	gp_pat	B'01111110'
	gp_pat	B'01000000'
	gp_pat	B'01111100'
	gp_pat	B'00000110'
	gp_pat	B'00000110'
	gp_pat	B'01100110'
	gp_pat	B'00111100'

	gp_pat	B'00011000'	;i
	gp_pat	B'00011000'
	gp_pat	B'00000000'
	gp_pat	B'00011000'
	gp_pat	B'00011000'
	gp_pat	B'00011000'
	gp_pat	B'00111100'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;6
	gp_pat	B'00001110'
	gp_pat	B'00011000'
	gp_pat	B'00110000'
	gp_pat	B'01111100'
	gp_pat	B'01100110'
	gp_pat	B'01100110'
	gp_pat	B'00111100'

	gp_pat	B'00000000'	;r
	gp_pat	B'00000000'
	gp_pat	B'00110110'
	gp_pat	B'00111000'
	gp_pat	B'00110000'
	gp_pat	B'00110000'
	gp_pat	B'00110000'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;7
	gp_pat	B'01111110'
	gp_pat	B'00000110'
	gp_pat	B'00001100'
	gp_pat	B'00011000'
	gp_pat	B'00110000'
	gp_pat	B'00110000'
	gp_pat	B'00110000'

	gp_pat	B'00000000'	;a
	gp_pat	B'00000000'
	gp_pat	B'00011100'
	gp_pat	B'00000110'
	gp_pat	B'00011110'
	gp_pat	B'00110110'
	gp_pat	B'00011110'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;8
	gp_pat	B'00111100'
	gp_pat	B'01100110'
	gp_pat	B'01100110'
	gp_pat	B'00111100'
	gp_pat	B'01100110'
	gp_pat	B'01100110'
	gp_pat	B'00111100'

	gp_pat	B'00101000'	;Sonne
	gp_pat	B'10111010'
	gp_pat	B'01000100'
	gp_pat	B'11000110'
	gp_pat	B'01000100'
	gp_pat	B'10111010'
	gp_pat	B'00101000'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;9
	gp_pat	B'00111100'
	gp_pat	B'01100110'
	gp_pat	B'01100110'
	gp_pat	B'00111110'
	gp_pat	B'00001100'
	gp_pat	B'00011000'
	gp_pat	B'01110000'

	gp_pat	B'00011110'	;Mond
	gp_pat	B'01100100'
	gp_pat	B'10001000'
	gp_pat	B'10001000'
	gp_pat	B'10001000'
	gp_pat	B'01100100'
	gp_pat	B'00011110'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;-
	gp_pat	B'00000000'
	gp_pat	B'00000000'
	gp_pat	B'00000000'
	gp_pat	B'01111110'
	gp_pat	B'00000000'
	gp_pat	B'00000000'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;rmisch 1
	gp_pat	B'00000000'
	gp_pat	B'01110000'
	gp_pat	B'00100000'
	gp_pat	B'00100000'
	gp_pat	B'00100000'
	gp_pat	B'01110000'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;unbenutzt
	gp_pat	B'00000000'
	gp_pat	B'00000000'
	gp_pat	B'00000000'
	gp_pat	B'00000000'
	gp_pat	B'00000000'
	gp_pat	B'00000000'
	gp_pat	B'00000000'

	gp_pat	B'00000000'	;rmisch 2
	gp_pat	B'00000000'
	gp_pat	B'01111100'
	gp_pat	B'00101000'
	gp_pat	B'00101000'
	gp_pat	B'00101000'
	gp_pat	B'01111100'
	gp_pat	B'00000000'

get_day:
	addwf	PCL		;Code fr Wochentag holen
	retlw	H'04'		;Mo
	retlw	H'15'		;Di
	retlw	H'05'		;Mi
	retlw	H'14'		;Do
	retlw	H'26'		;Fr
	retlw	H'37'		;Sa
	retlw	H'34'		;So

	if	($>>8) != (get_pattern>>8)
	error	"Sprungtabelle falsch"
	endif
;================================================================
;	Initialisierung
reset:
	bsf	STATUS,RP0
	movlw	TRIS_A
	movwf	TRISA		;Port A programmieren
	movlw	TRIS_B
	movwf	TRISB		;Port B programmieren
	movlw	OPT
	movwf	OPTION_REG	;Optionen programmieren
	bcf	STATUS,RP0
;----------------------------------------------------------------
	bsf	PORTA,MUX	;Ausgang schalten

	call	lcd_init	;LCD initialisieren

	movlw	get_pattern>>8
	movwf	PCLATH		;Sprnge vorbereiten

	clrf	flags
	movlw	N_TICKS
	movwf	ticks		;Sekundenticker zurcksetzen

	movf	second,w
	sublw	60
	btfss	STATUS,C
	goto	reset_clr
	movf	minute,w
	sublw	60-1
	btfss	STATUS,C
	goto	reset_clr
	movf	hour,w
	sublw	24-1
	btfss	STATUS,C
	goto	reset_clr
	movf	day,w
	sublw	7-1
	btfsc	STATUS,C
	goto	reset_quit	;Uhrzeit erhalten
reset_clr:
	movlw	60
	movwf	second
	clrf	minute
	clrf	hour
	clrf	day		;Uhrzeit lschen

reset_quit:
	clrf	TMR0
	bcf	INTCON,T0IF
	bsf	INTCON,T0IE
	bsf	INTCON,GIE	;Interrupts freigeben

	clrf	EEADR		;Schaltzeiger zurcksetzen
;================================================================
;	Hauptfunktionen
main_loop:
	bcf	flags,fl_colon	;Doppelpunkt blinkt

	movf	day,w
	movwf	dsp_buf+0
	rrf	EEADR,w
	andlw	B'0011'
	xorlw	B'0001'
	iorlw	B'1000'
	movwf	dsp_buf+1
	call	dsp_dayetc	;Tag und Schalter anzeigen

	movf	hour,w
	movwf	dsp_buf+0
	movf	minute,w
	movwf	dsp_buf+1
	call	dsp_time	;Uhrzeit anzeigen

	call	wait_action	;auf Aktion warten

	bsf	flags,fl_colon	;Doppelpunkt dauerhaft

	btfss	keys,KEY_ESC
	goto	edit_act
	btfss	keys,KEY_UP
	goto	edit_up
	btfss	keys,KEY_DN
	goto	edit_dn		;auf Tasten reagieren

	call	sw_get		;nchste Schaltzeit holen

	movf	day,w
	xorwf	sw_day,w
	btfss	STATUS,Z
	goto	main_loop	;Tag noch nicht erreicht?

	incf	sw_hour,w
	btfsc	STATUS,Z
	goto	main_nosw	;Zeit gelscht?

	movf	hour,w
	xorwf	sw_hour,w
	btfss	STATUS,Z
	goto	main_loop
	movf	minute,w
	xorwf	sw_minute,w
	btfss	STATUS,Z
	goto	main_loop	;Zeit noch nicht erreicht?

main_sw:
	btfsc	EEADR,1
	bsf	PORTA,MUX
	btfss	EEADR,1
	bcf	PORTA,MUX	;Ausgang schalten
main_nosw:
	movlw	2
	addwf	EEADR
	movf	EEADR,w
	xorlw	7*8
	btfsc	STATUS,Z
	clrf	EEADR		;Schaltzeiger zyklisch erhhen
	goto	main_loop
;================================================================
;	Aktuelle Zeit ndern
edit_act:
	movf	hour,w
	movwf	edit_buf+0
	movf	minute,w
	movwf	edit_buf+1
	movf	day,w
	movwf	edit_buf+2	;Zeit fr nderung vorbereiten
ea_loop:
	call	lcd_init	;LCD wiederholt initialisieren

	movlw	T_TIMEOUT+1
	movwf	timeout		;Eingabetimeout vorbereiten
ea_kloop:
	movf	edit_buf+2,w
	movwf	dsp_buf+0
	movlw	B'1100'
	movwf	dsp_buf+1
	call	dsp_dayetc	;Anzeige aktualisieren

	call	wait_nokey	;auf Aktion warten

	btfss	keys,KEY_ESC
	goto	ea_esc
	btfss	keys,KEY_UP
	goto	ea_up
	btfss	keys,KEY_DN
	goto	ea_dn
	btfss	keys,KEY_OK
	goto	ea_ok		;auf Tasten reagieren

	decf	timeout,w
	btfsc	STATUS,Z
	goto	ea_esc		;bei Eingabetimeout Abbruch

	movlw	(1<<fl_blink0)|(1<<fl_blink1)
	xorwf	flags		;Wochentag blinkt

	goto	ea_kloop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ea_up:
	incf	edit_buf+2
	movf	edit_buf+2,w
	xorlw	7
	btfsc	STATUS,Z
	clrf	edit_buf+2	;Wochentag zyklisch erhhen
	goto	ea_loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ea_dn:
	decf	edit_buf+2
	movlw	6
	btfsc	edit_buf+2,7
	movwf	edit_buf+2	;Wochentag zyklisch erniedrigen
	goto	ea_loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ea_esc:
	movlw	NO_BLINK
	andwf	flags		;Anzeige wieder vollstndig

	goto	main_loop
;----------------------------------------------------------------
ea_ok:
	movlw	NO_BLINK
	andwf	flags		;Anzeige wieder vollstndig

	movf	edit_buf+2,w
	movwf	dsp_buf+0
	movlw	B'1100'
	movwf	dsp_buf+1
	call	dsp_dayetc	;Anzeige aktualisieren

	clrf	edit_min+0
	clrf	edit_min+1
	movlw	23
	movwf	edit_max+0
	movlw	59
	movwf	edit_max+1
	bcf	flags,fl_sw
	call	edit_time	;auch Uhrzeit ndern

	btfsc	STATUS,C
	goto	ea_esc		;nderung abgebrochen?

	movlw	60
	movwf	second
	movf	edit_buf+0,w
	movwf	hour
	movf	edit_buf+1,w
	movwf	minute
	movf	edit_buf+2,w
	movwf	day		;neue Zeit speichern

	swapf	day,w
	movwf	EEADR
	bcf	STATUS,C
	rrf	EEADR		;Schaltzeiger wie Tag setzen

	bcf	PORTA,MUX	;Ausgang erstmal AUS1
	goto	ea_ok_loop

ea_ok_next:
	btfsc	EEADR,1
	bsf	PORTA,MUX
	btfss	EEADR,1
	bcf	PORTA,MUX	;Ausgang schalten

	movlw	2
	addwf	EEADR
	movf	EEADR,w
	xorlw	7*8
	btfsc	STATUS,Z
	clrf	EEADR		;Schaltzeiger zyklisch erhhen
ea_ok_loop:
	call	sw_get		;Schaltzeit holen

	movf	day,w
	xorwf	sw_day,w
	btfss	STATUS,Z
	goto	main_loop	;bereits nchster Tag?

	incf	sw_hour,w
	btfsc	STATUS,Z
	goto	ea_ok_next	;Schaltzeit gelscht?

	movf	hour,w
	subwf	sw_hour,w
	btfss	STATUS,C
	goto	ea_ok_next
	btfss	STATUS,Z
	goto	main_loop
	movf	minute,w
	subwf	sw_minute,w
	btfss	STATUS,C
	goto	ea_ok_next
	goto	main_loop	;nchste Schaltzeit suchen
;================================================================
;	Schaltspeicher auswhlen
edit_up:
	movf	EEADR,w
	movwf	sw_old+0	;aktuellen Zeiger retten
edit_up_loop:
	movlw	-2
	addwf	EEADR,w
	btfss	STATUS,C
	movlw	6*8+6
	movwf	EEADR		;Schaltzeiger zyklisch erniedr.
	goto	eud_loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
edit_dn:
	movf	EEADR,w
	movwf	sw_old+0	;aktuellen Zeiger retten
	goto	eud_loop

edit_dn_loop:
	movlw	2
	addwf	EEADR
	movf	EEADR,w
	xorlw	7*8
	btfsc	STATUS,Z
	clrf	EEADR		;Schaltzeiger zyklisch erhhen
;----------------------------------------------------------------
eud_loop:
	call	lcd_init	;LCD wiederholt initialisieren

	movlw	T_TIMEOUT+1
	movwf	timeout		;Eingabetimeout vorbereiten
eud_kloop:
	call	sw_get		;Schaltzeit holen

	movf	sw_day,w
	movwf	dsp_buf+0
	rrf	EEADR,w
	andlw	B'0011'
	movwf	dsp_buf+1
	call	dsp_dayetc	;Tag und Schalter anzeigen

	movf	sw_hour,w
	movwf	dsp_buf+0
	movf	sw_minute,w
	movwf	dsp_buf+1
	call	dsp_time	;Uhrzeit anzeigen

	call	wait_nokey	;auf Aktion warten

	btfss	keys,KEY_OK
	goto	main_sw
	btfss	keys,KEY_ESC
	goto	edit_sw
	btfss	keys,KEY_UP
	goto	edit_up_loop
	btfss	keys,KEY_DN
	goto	edit_dn_loop	;auf Tasten reagieren

	decf	timeout,w
	btfsc	STATUS,Z
	goto	es_quit		;bei Eingabetimeout Abbruch

	goto	eud_kloop
;----------------------------------------------------------------
edit_sw:
	movf	sw_hour,w
	movwf	edit_buf+0
	movf	sw_minute,w
	movwf	edit_buf+1	;Zeit fr nderung vorbereiten

	movf	EEADR,w
	movwf	sw_old+1	;Schaltzeiger retten

es_chk	macro	file,label
	local	es_goon

	movf	EEADR,w
	xorwf	sw_old+1,w
	btfsc	STATUS,Z
	goto	label		;Suche fertig?

	call	sw_get
	incf	sw_hour,w
	btfsc	STATUS,Z
	goto	es_goon		;Zeit gelscht?
	movf	sw_hour,w
	movwf	file+0
	movf	sw_minute,w
	movwf	file+1		;Zeit merken
es_goon:
	endm

	clrf	edit_min+0
	clrf	edit_min+1
	bcf	EEADR,1
	bcf	EEADR,2
	es_chk	edit_min,es_max		;Test fr "Ein 1"
	bsf	EEADR,1
	es_chk	edit_min,es_max		;Test fr "Aus 1"
	bcf	EEADR,1
	bsf	EEADR,2
	es_chk	edit_min,es_max		;Test fr "Ein 2"

es_max:
	movlw	23
	movwf	edit_max+0
	movlw	59
	movwf	edit_max+1
	bsf	EEADR,1
	bsf	EEADR,2
	es_chk	edit_max,es_edit	;Test fr "Aus 2"
	bcf	EEADR,1
	es_chk	edit_max,es_edit	;Test fr "Ein 2"
	bsf	EEADR,1
	bcf	EEADR,2
	es_chk	edit_max,es_edit	;Test fr "Aus 1"

es_edit:
	movf	sw_old+1,w
	movwf	EEADR		;Schaltzeiger wiederholen

	bsf	flags,fl_sw
	call	edit_time	;Uhrzeit ndern

	btfsc	STATUS,C
	goto	es_quit		;nderung abgebrochen?

	movf	edit_buf+0,w
	movwf	sw_hour
	movf	edit_buf+1,w
	movwf	sw_minute
	call	sw_put		;neue Zeit speichern
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
es_quit:
	movf	sw_old+0,w
	movwf	EEADR		;Schaltzeiger zurcksetzen
	goto	main_loop
;================================================================
;	Uhrzeit allgemein ndern
edit_time:
eth_loop:
	call	lcd_init	;LCD wiederholt initialisieren

	movlw	T_TIMEOUT+1
	movwf	timeout		;Eingabetimeout vorbereiten
eth_kloop:
	movf	edit_buf+0,w
	movwf	dsp_buf+0
	movf	edit_buf+1,w
	movwf	dsp_buf+1
	call	dsp_time	;Anzeige aktualisieren

	call	wait_nokey	;auf Aktion warten

	btfss	keys,KEY_ESC
	goto	et_esc
	btfss	keys,KEY_UP
	goto	eth_up
	btfss	keys,KEY_DN
	goto	eth_dn
	btfss	keys,KEY_OK
	goto	eth_ok		;auf Tasten reagieren

	decf	timeout,w
	btfsc	STATUS,Z
	goto	et_esc		;bei Eingabetimeout Abbruch

 	movlw	(1<<fl_blink0)|(1<<fl_blink1)
	xorwf	flags		;Stunde blinkt

	goto	eth_kloop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
eth_up:
	incf	edit_buf+0
	call	et_check_max	;Stunde begrenzt erhhen
	goto	eth_loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
eth_dn:
	decf	edit_buf+0
	call	et_check_min	;Stunde begrenzt erniedrigen
	goto	eth_loop
;----------------------------------------------------------------
eth_ok:
	movlw	NO_BLINK
	andwf	flags		;Anzeige wieder vollstndig

et10_loop:
	call	lcd_init	;LCD wiederholt initialisieren

	movlw	T_TIMEOUT+1
	movwf	timeout		;Eingabetimeout vorbereiten
et10_kloop:
	movf	edit_buf+0,w
	movwf	dsp_buf+0
	movf	edit_buf+1,w
	movwf	dsp_buf+1
	call	dsp_time	;Anzeige aktualisieren

	call	wait_nokey	;auf Aktion warten

	btfss	keys,KEY_ESC
	goto	et_esc
	btfss	keys,KEY_UP
	goto	et10_up
	btfss	keys,KEY_DN
	goto	et10_dn
	btfss	keys,KEY_OK
	goto	et10_ok		;auf Tasten reagieren

	decf	timeout,w
	btfsc	STATUS,Z
	goto	et_esc		;bei Eingabetimeout Abbruch

	movlw	1<<fl_blink2
	xorwf	flags		;Minutenzehner blinkt

	goto	et10_kloop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
et10_up:
	movlw	10
	addwf	edit_buf+1
	call	et_check_max	;Minutenzehner begrenzt erhhen
	goto	et10_loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
et10_dn:
	movlw	10
	subwf	edit_buf+1
	call	et_check_min	;Minutenzehner begrenzt erniedr.
	goto	et10_loop
;----------------------------------------------------------------
et10_ok:
	bcf	flags,fl_blink2	;Anzeige wieder vollstndig
et1_loop:
	call	lcd_init	;LCD wiederholt initialisieren

	movlw	T_TIMEOUT+1
	movwf	timeout		;Eingabetimeout vorbereiten
et1_kloop:
	movf	edit_buf+0,w
	movwf	dsp_buf+0
	movf	edit_buf+1,w
	movwf	dsp_buf+1
	call	dsp_time	;Anzeige aktualisieren

	call	wait_nokey	;auf Aktion warten

	btfss	keys,KEY_ESC
	goto	et_esc
	btfss	keys,KEY_UP
	goto	et1_up
	btfss	keys,KEY_DN
	goto	et1_dn
	btfss	keys,KEY_OK
	goto	et1_ok		;auf Tasten reagieren

	decf	timeout,w
	btfsc	STATUS,Z
	goto	et_esc		;bei Eingabetimeout Abbruch

	movlw	1<<fl_blink3
	xorwf	flags		;Minuteneiner blinkt

	goto	et1_kloop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
et1_up:
	incf	edit_buf+1
	call	et_check_max	;Minuteneiner begrenzt erhhen
	goto	et1_loop
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
et1_dn:
	decf	edit_buf+1
	call	et_check_min	;Minuteneiner begrenzt erniedr.
	goto	et1_loop
;----------------------------------------------------------------
et1_ok:
	movlw	NO_BLINK
	andwf	flags		;Anzeige wieder vollstndig

	bcf	STATUS,C	;nderung akzeptiert
	return
;----------------------------------------------------------------
et_esc:
	movlw	NO_BLINK
	andwf	flags		;Anzeige wieder vollstndig

	bsf	STATUS,C	;abgebrochen
	return
;----------------------------------------------------------------
;	Zeit gegen obere Grenze begrenzen
et_check_max:
	btfss	flags,fl_sw
	goto	etc_max_nosw	;keine Schaltzeit?

	call	et_check_min
	incf	edit_buf+0,w
	btfsc	STATUS,Z
	call	etc_min_nosw	;bei Bedarf buf auf min setzen

etc_max_nosw:
	movlw	60
	subwf	edit_buf+1,w
	btfss	STATUS,C
	goto	etc_max_goon

	incf	edit_buf+0
	movwf	edit_buf+1	;Minutenbertrag bercksichtigen

etc_max_goon:
	movf	edit_max+0,w
	subwf	edit_buf+0,w
	btfss	STATUS,C
	return			;buf < max ?
	btfss	STATUS,Z
	goto	etc_max_over	;buf > max ?

	movf	edit_max+1,w
	subwf	edit_buf+1,w
	btfss	STATUS,C
	return			;buf < max ?
etc_max_over:
	movf	edit_max+0,w
	movwf	edit_buf+0
	movf	edit_max+1,w
	movwf	edit_buf+1	;buf auf max setzen
	return
;----------------------------------------------------------------
;	Zeit gegen untere Grenze begrenzen
et_check_min:
	movlw	60
	addwf	edit_buf+1,w
	btfss	STATUS,C
	goto	etc_min_goon

	decf	edit_buf+0
	movwf	edit_buf+1	;Minutenbertrag bercksichtigen

etc_min_goon:
	btfsc	edit_buf+0,7
	goto	etc_min_over	;buf < 0 ?

	movf	edit_buf+0,w
	subwf	edit_min+0,w
	btfss	STATUS,C
	return			;buf > min ?
	btfss	STATUS,Z
	goto	etc_min_over	;buf < min ?

	movf	edit_buf+1,w
	subwf	edit_min+1,w
	btfss	STATUS,C
	return			;buf > min ?
etc_min_over:
	btfss	flags,fl_sw
	goto	etc_min_nosw	;keine Schaltzeit?

	movlw	H'FF'
	movwf	edit_buf+0
	movwf	edit_buf+1	;buf auf "aus" setzen
	return
etc_min_nosw:
	movf	edit_min+0,w
	movwf	edit_buf+0
	movf	edit_min+1,w
	movwf	edit_buf+1	;buf auf min setzen
	return
;================================================================
;	Tag usw. ausgeben
dsp_dayetc:
	movf	dsp_buf+0,w
	call	get_day
	movwf	dsp_buf+0	;Wochentag umwandeln

	movlw	H'8A'
	btfsc	dsp_buf+1,0
	iorlw	H'10'
	btfsc	dsp_buf+1,1
	iorlw	H'01'
	btfsc	dsp_buf+1,2
	bsf	flags,fl_blink2
	btfsc	dsp_buf+1,3
	bsf	flags,fl_blink3
	movwf	dsp_buf+1	;Schaltanzeige umwandeln

	movlw	8
	movwf	dsp_cnt		;Schleife vorbereiten

	movlw	LCD_DSP_ADDR+H'20'
	movwf	lcd_addr
	call	dsp_half	;erste Hlfte ausgeben

	movlw	LCD_DSP_ADDR+H'00'
	movwf	lcd_addr
	call	dsp_half	;zweite Hlfte ausgeben

	bcf	flags,fl_blink2
	bcf	flags,fl_blink3
	return
;================================================================
;	Uhrzeit ausgeben
dsp_time:
	movf	dsp_buf+0,w
	call	bin2bcd
	movwf	dsp_buf+0
	movf	dsp_buf+1,w
	call	bin2bcd
	movwf	dsp_buf+1	;Zeit umwandeln

	clrf	dsp_cnt		;Schleife vorbereiten

	movlw	LCD_DSP_ADDR+H'30'
	movwf	lcd_addr
	call	dsp_half	;erste Hlfte ausgeben

	movlw	LCD_DSP_ADDR+H'10'
	movwf	lcd_addr
	goto	dsp_half	;zweite Hlfte ausgeben
;================================================================
;	Hlfte einer Zeile ausgeben
dsp_half:
	call	lcd_start
	movf	lcd_addr,w
	call	lcd_byte	;Zeiger im LCD setzen
dh_loop:
	movf	dsp_buf+0,w
	andlw	H'F0'
	addwf	dsp_cnt,w
	call	get_pattern
	movwf	lcd_line+0
	rlf	lcd_line+0
	rlf	lcd_line+0
	bcf	lcd_line+0,1
	btfsc	STATUS,C
	bsf	lcd_line+0,1	;erste Stelle

	swapf	dsp_buf+0,w
	andlw	H'F0'
	addwf	dsp_cnt,w
	call	get_pattern
	movwf	lcd_line+1
	rlf	lcd_line+1
	rlf	lcd_line+1
	bcf	lcd_line+1,1
	btfsc	STATUS,C
	bsf	lcd_line+1,1	;zweite Stelle

	movf	dsp_buf+1,w
	andlw	H'F0'
	addwf	dsp_cnt,w
	call	get_pattern
	movwf	lcd_line+2
	rrf	lcd_line+2
	rrf	lcd_line+2
	bcf	lcd_line+2,6
	btfsc	STATUS,C
	bsf	lcd_line+2,6	;dritte Stelle

	swapf	dsp_buf+1,w
	andlw	H'F0'
	addwf	dsp_cnt,w
	call	get_pattern
	movwf	lcd_line+3
	rrf	lcd_line+3
	rrf	lcd_line+3
	bcf	lcd_line+3,6
	btfsc	STATUS,C
	bsf	lcd_line+3,6	;vierte Stelle

	btfsc	flags,fl_blink0
	clrf	lcd_line+0
	btfsc	flags,fl_blink1
	clrf	lcd_line+1
	btfsc	flags,fl_blink2
	clrf	lcd_line+2
	btfsc	flags,fl_blink3
	clrf	lcd_line+3	;bei Bedarf blinken

	btfsc	lcd_addr,4
	btfss	dsp_cnt,1
	goto	dh_out		;"falsche" Pixelzeile?
	btfsc	second,0
	goto	dh_colon
	btfss	flags,fl_colon
	goto	dh_out		;blinkend/statisch?
dh_colon:
	bsf	lcd_line+1,0
	bsf	lcd_line+2,7	;Doppelpunkt dazu

dh_out:
	call	lcd_putl	;Pixelzeile ausgeben

	incf	dsp_cnt
	btfss	dsp_cnt,0
	btfsc	dsp_cnt,1
	goto	dh_loop		;noch nicht fertig?

	return
;================================================================
;	Binr (FF/00..3F) zu BCD (AA/00..63) umwandeln
bin2bcd:
	movwf	b2b_buf		;Binrzahl merken

	incf	b2b_buf,w
	btfsc	STATUS,Z
	retlw	H'AA'		;Spezialfall "FF"

	swapf	b2b_buf
	rlf	b2b_buf,w
	rlf	b2b_buf
	movlw	H'03'
	addwf	b2b_buf
	btfss	b2b_buf,3
	subwf	b2b_buf
	rlf	b2b_buf,w
	rlf	b2b_buf
	movlw	H'03'
	addwf	b2b_buf
	btfss	b2b_buf,3
	subwf	b2b_buf
	rlf	b2b_buf,w
	rlf	b2b_buf
	movlw	H'03'
	addwf	b2b_buf
	btfss	b2b_buf,3
	subwf	b2b_buf
	rlf	b2b_buf,w
	rlf	b2b_buf		;umwandeln

	movf	b2b_buf,w	;BCD-Zahl holen
	return
;================================================================
;	Auf Benutzer oder Zeit warten
wait_nokey:
	bsf	PORTA,LCD_CLK
	movf	keys,w
	xorlw	KEY_ALL
	bcf	PORTA,LCD_CLK
	btfss	STATUS,Z
	goto	wait_nokey	;auf Loslassen warten
;----------------------------------------------------------------
wait_action:
	bcf	flags,fl_action	;Aktion beendet
wa_loop:
	bsf	PORTA,LCD_CLK
	btfsc	flags,fl_action
	return			;Aktion ntig?
	bcf	PORTA,LCD_CLK
	goto	wa_loop		;Takt an LCD, solange nichts los

LCD_DIVIDER	set	0
DIVIDER		set	F_OSC/4/($-wa_loop+1)	;Frequenz LCD_CLK
DIVIDER		set	DIVIDER/16/16		;LCD-Teiler
DIVIDER		set	DIVIDER/F_LCD		;"Bild"-Frequenz
	if	DIVIDER>16*16
LCD_DIVIDER	set	LCD_DIVIDER|H'80'
DIVIDER		set	DIVIDER/16
	endif
	if	DIVIDER>32*4
LCD_DIVIDER	set	LCD_DIVIDER|H'40'
DIVIDER		set	DIVIDER/4
	endif
	if	DIVIDER>32*2
LCD_DIVIDER	set	LCD_DIVIDER|H'20'
DIVIDER		set	DIVIDER/2
	endif
	if	(DIVIDER>32) || (DIVIDER<4)
	error	"Fehler bei LCD_DIVIDER"
	endif
LCD_DIVIDER	set	LCD_DIVIDER+DIVIDER-1	;Teiler berechnen
;================================================================
;	LCD initialisieren
lcd_init:
	call	lcd_start
	movlw	LCD_DIV_ADDR
	call	lcd_byte
	movlw	LCD_DIVIDER
	goto	lcd_byte	;Teiler einstellen
;----------------------------------------------------------------
;	Pixelzeile an LCD senden
lcd_putl:
	rlf	lcd_line+0,w	;0h..0l.. 2h..2l..
	andlw	B'10101010'
	xorwf	lcd_line+2	;0h..0l.. 2h0l2l..
	rrf	lcd_line+2,w
	andlw	B'01010101'
	xorwf	lcd_line+0	;0h..2h.. 2h0l2l..
	rlf	lcd_line+0,w
	andlw	B'10101010'
	xorwf	lcd_line+2	;0h..2h.. 0l..2l..
	rlf	lcd_line+1,w	;1h..1l.. 3h..3l..
	andlw	B'10101010'
	xorwf	lcd_line+3	;1h..1l.. 3h1l3l..
	rrf	lcd_line+3,w
	andlw	B'01010101'
	xorwf	lcd_line+1	;1h..3h.. 3h1l3l..
	rlf	lcd_line+1,w
	andlw	B'10101010'
	xorwf	lcd_line+3	;1h..3h.. 1l..3l..

	movf	lcd_line+3,w
	call	lcd_byte
	movf	lcd_line+1,w
	call	lcd_byte
	movf	lcd_line+2,w
	call	lcd_byte
	movf	lcd_line+0,w
	goto	lcd_byte	;Pixelzeile ausgeben
;----------------------------------------------------------------
;	Startbyte an LCD senden
lcd_start:
	bcf	flags,fl_lcd_par	;gerade Paritt
	movlw	0
	goto	lcd_txd		;Wert "0"
;----------------------------------------------------------------
;	Byte an LCD senden
lcd_byte:
	bsf	flags,fl_lcd_par	;ungerade Paritt
;----------------------------------------------------------------
;	An LCD senden
lcd_txd:
	bcf	PORTA,LCD_DAT
	bsf	PORTA,LCD_CLK
	bcf	PORTA,LCD_CLK	;Startbit

	movwf	lcd_buf
	movlw	8
	movwf	lcd_cnt		;Schleife vorbereiten
lt_loop:
	movlw	1<<fl_lcd_par
	btfsc	lcd_buf,0
	xorwf	flags		;Paritt mitfhren

	bcf	PORTA,LCD_DAT
	btfsc	lcd_buf,0
	bsf	PORTA,LCD_DAT
	bsf	PORTA,LCD_CLK
	bcf	PORTA,LCD_CLK	;Bit senden

	rrf	lcd_buf		;nchstes Bit

	decfsz	lcd_cnt
	goto	lt_loop		;noch nicht alle Bits?

	bcf	PORTA,LCD_DAT
	btfsc	flags,fl_lcd_par
	bsf	PORTA,LCD_DAT
	bsf	PORTA,LCD_CLK
	bcf	PORTA,LCD_CLK	;Parittsbit

	bsf	PORTA,LCD_DAT
	bsf	PORTA,LCD_CLK
	bcf	PORTA,LCD_CLK
	bsf	PORTA,LCD_CLK
	bcf	PORTA,LCD_CLK	;2 Stopbits
	return
;================================================================
;	Schaltzeit holen
sw_get:
	bsf	STATUS,RP0
	bsf	EECON1,RD
	bcf	STATUS,RP0
	movf	EEDATA,w
	movwf	sw_hour
	incf	EEADR
	bsf	STATUS,RP0
	bsf	EECON1,RD
	bcf	STATUS,RP0
	movf	EEDATA,w
	movwf	sw_minute
	decf	EEADR		;Schaltzeit holen

	swapf	EEADR,w
	andlw	H'83'
	movwf	sw_day
	rlf	sw_day,w
	rlf	sw_day		;Schalttag berechnen
	return
;================================================================
;	Schaltzeit speichern
sw_put:
	bcf	INTCON,T0IE	;Interrupts sperren

	movf	sw_hour,w
	movwf	EEDATA
	bsf	STATUS,RP0
	bsf	EECON1,WREN
	bcf	EECON1,EEIF
	movlw	H'55'
	movwf	EECON2
	movlw	H'AA'
	movwf	EECON2
	bsf	EECON1,WR
sp_loop1:
	btfss	EECON1,EEIF
	goto	sp_loop1
	bcf	STATUS,RP0
	incf	EEADR

	bsf	INTCON,T0IE
	nop
	nop
	bcf	INTCON,T0IE	;bei Bedarf Interrupt

	movf	sw_minute,w
	movwf	EEDATA
	bsf	STATUS,RP0
	bcf	EECON1,EEIF
	movlw	H'55'
	movwf	EECON2
	movlw	H'AA'
	movwf	EECON2
	bsf	EECON1,WR
sp_loop2:
	btfss	EECON1,EEIF
	goto	sp_loop2
	bcf	EECON1,WREN
	bcf	STATUS,RP0
	decf	EEADR		;Schaltzeit speichern

	bsf	INTCON,T0IE	;Interrupts freigeben
	return
;================================================================
	end
